//The sample code for driving one way motor encoder
#include <PID_v1.h>

//****Ports de commande du moteur A et B supplied with 6 volt= 90 round/min

//MOTOR A  encoderA+motorA pin 3,7,9,8,5 set with myPIDA

#define encoderApinA 3  //the interrupt pin 1 on Arduino Uno
#define encoderApinB 7  // to detect the direction

#define motorPinA1  8 // pin to set H BRIDGE of L298N  
#define motorPinA2  9 // pin to set H BRIDGE of L298N  
#define enablePinA  5 // Commande de vitesse moteur, to Output ENA pour L298 the second

//MOTEUR B  encoderB+motorB pin 2,4,11,10,6 set with myPIDB
// moteur gauche  
#define encoderBpinA  2 //the interrupt pin 0 in Arduino Uno
#define encoderBpinB  4  // to detect the direction

#define motorPinB1  10 // pin to set H BRIDGE of L298N
#define motorPinB2  11 // pin to set H BRIDGE of L298N
#define enablePinB  6  // Commande de vitesse moteur, to Output ENA pour L298 the second

//****VARIABLE TO CONTROL MOTOR A with myPID A and MOTOR B with myPID B
//MOTOR A (Left)

volatile byte encoderAPinALast;  // the last time detected

boolean DirectionA;//the rotation direction 


double  latestDurationCountA,durationA,abs_durationA; //speed detected of wheelA
double val_outputA;//Power supplied to the motor PWM value.
double SetpointA;// Set the speed of wheelA
double Kp=1, Ki=10, Kd=0.01; // 1 ; 10 ; 0.01
PID myPIDA(&abs_durationA, &val_outputA, &SetpointA, Kp, Ki, Kd, DIRECT); 

//MOTOR B (Right)

volatile byte encoderBPinALast;

boolean DirectionB;//the rotation direction 


double latestDurationCountB, durationB, abs_durationB; //speed detected of wheelB
double val_outputB;//Power supplied to the motor PWM value.
double SetpointB;
//double Kp=0.6, Ki=5, Kd=0; // i keep the same tuning for both motors
PID myPIDB(&abs_durationB, &val_outputB, &SetpointB, Kp, Ki, Kd, DIRECT); 

//Time to stop motors

unsigned long time0 = 0; // stock un temps à un instant donné
unsigned long timer = 0; // variable qui va contenir le dernier temps enregistré via millis

// Setpoint in the Setup

int8_t valeur = 20;  //lecture de la consigne via le moniteur serie pour test, initialisé à 20 pour test rapide du PID

void setup()

{  
  Serial.begin(19200);//Initialize the serial port
  
   // Configuration des ports en mode "sortie" pour moteur A (Gauche)
  pinMode(motorPinA1, OUTPUT);
  pinMode(motorPinA2, OUTPUT);
  pinMode(enablePinA, OUTPUT);

   // Configuration des ports en mode "sortie" pour moteur B (Droit)
   
   pinMode(motorPinB1, OUTPUT);   //L298N Control port settings direction of motor B (originaly L298N)
   pinMode(motorPinB2, OUTPUT);  //L298N Control port settings direction of motor B
   pinMode(enablePinB, OUTPUT);  // powerRate to control speed of motor B
   
   // REGLAGE PID MOTOR A (gauche)
   SetpointA; // Set Speed to MOTOR A
   
   myPIDA.SetMode(AUTOMATIC);//PID is set to automatic mode
   myPIDA.SetSampleTime(5);//Set PID sampling frequency is 5ms
  
   EncoderInitA();// to initilizate the module count and direction of the wheelA ( moteur Gauche)
  

   //REGLAGE PID MOTEUR B (droit)
   
   SetpointB; 
   
   myPIDB.SetMode(AUTOMATIC);//PID is set to automatic mode
   myPIDB.SetSampleTime(5);//Set PID sampling frequency is 5ms
 
   EncoderInitB();// to initilizate the module count and direction of the wheelB 
}
 
void loop()

{ if (Serial.available()) {
    time0 = timer;
    valeur = Serial.parseInt(); //récupération des caractères sur le port série
    Serial.println("valeur");Serial.println(valeur);
    
  }
  if (valeur != 0) {
    commandMotor(valeur,valeur);
  }
  else {
    SetpointA=0; SetpointB=0;
    stopMotors();
  }
  timer = millis();
  
  if ( (timer - time0) >120000) { // Timeout !
    valeur = 0 ;
    stopMotors();
    time0 = timer; 

 }  


}
 
void EncoderInitA()
{
  DirectionA = true;//default -> Forward  
  pinMode(encoderApinB,INPUT);  
  attachInterrupt(3, wheelSpeedA, CHANGE);
}
 
void wheelSpeedA()
{
volatile byte LstateA = digitalRead(encoderApinA);  
  if((encoderAPinALast == LOW) && LstateA==HIGH)
  {
 volatile byte valA = digitalRead(encoderApinB);
    if(valA == LOW && DirectionA)
    {
      DirectionA = false; //Reverse
    }
    else if(valA == HIGH && !DirectionA)
    {
      DirectionA = true;  //Forward
    }
  }
  encoderAPinALast = LstateA; 
  
 
  if(!DirectionA)  durationA++;
  else  durationA--;

}
void advanceA()//Motor Forward
{
     digitalWrite(motorPinA1,HIGH);
     digitalWrite(motorPinA2,LOW);
     
     analogWrite(enablePinA,val_outputA);
}
void backA()//Motor reverse
{
      digitalWrite(motorPinA1,LOW);
     digitalWrite(motorPinA2,HIGH);
     
     analogWrite(enablePinA,val_outputA);
}

void StopA()//Motor stops
{
     digitalWrite(enablePinA, LOW); 
}

void EncoderInitB()
{
  DirectionB = true;//default -> Forward  
  pinMode(encoderBpinB,INPUT);  
  attachInterrupt(2, wheelSpeedB, CHANGE);
}
 
void wheelSpeedB()
{
  volatile byte LstateB = digitalRead(encoderBpinA); 
  if((encoderBPinALast == LOW) && LstateB==HIGH)
  {
   volatile byte valB = digitalRead(encoderBpinB);
    if(valB == LOW && DirectionB)
    {
      DirectionB = false; //Reverse
    }
    else if(valB == HIGH && !DirectionB)
    {
      DirectionB = true;  //Forward
    }
  }
  encoderBPinALast = LstateB;
 
  if(!DirectionB)  durationB++;
  else  durationB--;

}
void advanceB()//Motor Forward
{
     digitalWrite(motorPinB1,HIGH);
     digitalWrite(motorPinB2,LOW);
     
     analogWrite(enablePinB,val_outputB);
}
void backB()//Motor reverse
{
      digitalWrite(motorPinB1,LOW);
     digitalWrite(motorPinB2,HIGH);
     
     analogWrite(enablePinB,val_outputB);
}

void StopB()//Motor stops
{
     digitalWrite(enablePinB, LOW); 
    
}

void stopMotors() {

  StopB();
  StopA();

}
void commandMotor (int8_t commandeMoteurDroit, int8_t commandeMoteurGauche) {

if (valeur >0){
  
  SetpointB = commandeMoteurDroit;
  SetpointA = commandeMoteurGauche;

  }

  else  {

   SetpointB =  -commandeMoteurDroit;
   SetpointA =  -commandeMoteurGauche;
} 

//  SETTING SPEED MOTOR A (Left)  with myPIDA

//  declancher le comptage du temps si interruptions

      noInterrupts();
      latestDurationCountA = durationA;
      interrupts();
      abs_durationA = abs(latestDurationCountA);

       //  SetpointA=20; // set as default
        
   

      myPIDA.Compute();

      durationA = 0; //Count clear, wait for the next count
      
      
        Serial.print("LD"); Serial.print(latestDurationCountA); 
        Serial.print("D  : ");  Serial.print(durationA); Serial.print(durationA); Serial.print(durationA);

        Serial.print("V"); Serial.println(val_outputA);// volt to MOTOR= real speed
        
        // *******************************************

  // SETTING SPEED MOTEUR B (Right) with myPIDB (plus rapide, dephasage de 90 ° sur 2 minutes) 
  // motor B goes a little bit faster
   
      noInterrupts();
      latestDurationCountB = durationB;
      interrupts();
      abs_durationB = abs(latestDurationCountB);

     //  SetpointB=20; //  set as default
  
      myPIDB.Compute();//

       durationB = 0; //Count clear, wait for the next count
       
        Serial.print("LDb"); Serial.print(latestDurationCountB);  
        Serial.print("Db: ");  Serial.print(durationB);Serial.print(durationB);Serial.print(durationB);
       
        Serial.print("Vb"); Serial.println(val_outputB);// volt to MOTOR= real speed
  
  if (valeur >= 0) {
    advanceB();
  }
  if (valeur < 0) {
    backB();
  }
  if (valeur >= 0) {
    advanceA();
  }
  if (valeur < 0) {
    backA();
  }

}
